#include <stdio.h>
#include <time.h>
#include "string.h"
#include "nvs_flash.h"
#include "esp_system.h"
#include "esp_log.h"
#include "esp_peripherals.h"
#include "esp_spi_flash.h"
#include "esp_console.h"
#include "audio_mem.h"
#include "esp_heap_caps.h"
#include "esp_jpeg_dec.h"
#include "esp_jpeg_common.h"

jpeg_error_t esp_jpeg_decoder_one_picture_block_out(unsigned char* input_buf, int len, unsigned char** output_buf, int* out_len, int out_type, int rotation)
{
    unsigned char *output_block = NULL;
    jpeg_error_t ret = JPEG_ERR_OK;
    jpeg_dec_io_t* jpeg_io = NULL;
    jpeg_dec_header_info_t *out_info = NULL;

    // Generate configuration
    jpeg_dec_config_t config = DEFAULT_JPEG_DEC_CONFIG();
    config.output_type = JPEG_COLOR_SPACE_RGB888;
    config.rotate = JPEG_ROTATE_0D;
    config.scale.width = 0;
    config.scale.height = 0;
    config.clipper.width = 0;
    config.clipper.height = 0;
    config.block_enable = 1;

    // Empty handle to jpeg_decoder
    jpeg_dec_handle_t jpeg_dec = NULL;
    // Create jpeg_dec
    jpeg_dec = jpeg_dec_open(&config);
    if (jpeg_dec == NULL) {
        ret = JPEG_ERR_PAR;
        ESP_LOGE(TAG, "JPEG open error");
        goto jpeg_dec_failed;
    }

    // Create io_callback handle
    jpeg_io = calloc(1, sizeof(jpeg_dec_io_t));
    if (jpeg_io == NULL) {
        ret = JPEG_ERR_MEM;
        goto jpeg_dec_failed;
    }

    // Create out_info handle
    out_info = calloc(1, sizeof(jpeg_dec_header_info_t));
    if (out_info == NULL)
    {
        ret = JPEG_ERR_MEM;
        goto jpeg_dec_failed;
    }

    // Set input buffer and buffer len to io_callback
    jpeg_io->inbuf = input_buf;
    jpeg_io->inbuf_len = len;

    // Parse jpeg picture header and get picture for user and decoder
    ret = jpeg_dec_parse_header(jpeg_dec, jpeg_io, out_info);
    if (ret != JPEG_ERR_OK) {
        ESP_LOGE(TAG, "JPEG parse header error, ret = %d", ret);
        goto jpeg_dec_failed;
    }

    // Malloc output block buffer
    int output_len = 0;
    if (config.output_type == JPEG_COLOR_SPACE_RGB565_LE
        || config.output_type == JPEG_COLOR_SPACE_RGB565_BE
        || config.output_type == JPEG_COLOR_SPACE_CbYCrY) {
        output_len = out_info->width * (out_info->y_factory[0] << 3) * 2;
    }
    else if (config.output_type == JPEG_COLOR_SPACE_RGB888) {
        output_len = out_info->width * (out_info->y_factory[0] << 3) * 3;
    }
    output_block = jpeg_malloc_align(output_len, 16);
    if (output_block == NULL) {
        ret = JPEG_ERR_MEM;
        ESP_LOGE(TAG, "Output_block malloc fail, line: %d", __LINE__);
        return ret;
    }
    jpeg_io->outbuf = output_block;

    // Save to sdcard
    // FILE *f_out = fopen("/sdcard/dec_out.bin", "wb");
    // if (f_out == NULL) {
    //     ESP_LOGE(TAG, "File open fail");
    //     ret = JPEG_ERR_FAIL;
    //     goto jpeg_dec_failed;
    // }

    // Decode jpeg data
    // Each time output (jpeg_io->cur_line) line, (jpeg_io->cur_len) bytes
    while (jpeg_io->output_line < jpeg_io->output_height) {
        ret = jpeg_dec_process(jpeg_dec, jpeg_io);
        if (ret != JPEG_ERR_OK) {
            ESP_LOGE(TAG, "JPEG process error, ret = %d, line: %d", ret, __LINE__);
            goto jpeg_dec_failed;
        }
        
        // Print output info
        // printf("output line/height: %d/%d, cur_height: %d, cur_len: %d, addr: %p\n", jpeg_io->output_line, jpeg_io->output_height, jpeg_io->cur_line, jpeg_io->cur_len, jpeg_io->outbuf);
        
        // Use output data
        // Save to sdcard
        // fwrite(jpeg_io->outbuf, 1, jpeg_io->cur_len, f_out);
    }
    // Save to sdcard
    // fclose(f_out);

    // Decoder deinitialize
jpeg_dec_failed:
    jpeg_dec_close(jpeg_dec);
    if (jpeg_io) {
        free(jpeg_io);
    }
    if (out_info) {
        free(out_info);
    }
    jpeg_free_align(output_block);
    return ret;
}

static esp_err_t use_jpeg_dec(void)
{
    unsigned char *jpeg_buffer = /* input buffer */;
    int len = /* input buffer length */;

    int ret = esp_jpeg_decoder_one_picture(jpeg_buffer, len);
    if (ret != ESP_OK) {
        ESP_LOGE(TAG, "JPEG dec error, ret = %d", ret);
    }

    return ESP_OK;
}